package com.muyu.common.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.common.domain.req.RuleDataAddReq;
import com.muyu.common.domain.req.RuleDataUpdReq;
import lombok.*;
import lombok.experimental.SuperBuilder;

import java.util.function.Supplier;

/**
 * @Author：yang
 * @Package：com.muyu.common.domain
 * @Project：cloud-integration
 * @name：RuleDataClassify
 * @Date：2024/8/21 20:25
 */
@EqualsAndHashCode(callSuper = true)
@Data
@SuperBuilder
@AllArgsConstructor
@NoArgsConstructor
@TableName("rule_data")
public class RuleData extends BaseEntity {

    /**
     * 规则ID
     */
    @TableId(value = "rule_id",type = IdType.AUTO)
    private Long ruleId;
    /**
     * 规则分类
     */
    private Integer ruleClassify;
    /**
     * 规则名称
     */
    private String ruleName;
    /**
     * 规则类型
     */
    private String ruleType;
    /**
     * 规则作用域
     */
    private String ruleRegion;
    /**
     * 引擎编码
     */
    private String ruleCoding;
    /**
     * 激活状态
     */
    private String ruleActivate;
    /**
     * 规则状态
     */
    private String ruleStatus;
    /**
     * 规则描述
     */
    private String ruleDescribe;

    public static RuleData updBuild(RuleDataUpdReq ruleDataUpdReq, Supplier<Long> idSupplier) {
        return RuleData.builder()
                .ruleId(idSupplier.get())
                .ruleClassify(ruleDataUpdReq.getRuleClassify())
                .ruleName(ruleDataUpdReq.getRuleName())
                .ruleType(ruleDataUpdReq.getRuleType())
                .ruleRegion(ruleDataUpdReq.getRuleRegion())
                .ruleCoding(ruleDataUpdReq.getRuleCoding())
                .ruleActivate(ruleDataUpdReq.getRuleActivate())
                .ruleStatus(ruleDataUpdReq.getRuleStatus())
                .ruleDescribe(ruleDataUpdReq.getRuleDescribe())
                .build();
    }

    public static RuleData addBuild(RuleDataAddReq ruleDataAddReq) {
        return RuleData.builder()
                .ruleName(ruleDataAddReq.getRuleName())
                .ruleType(ruleDataAddReq.getRuleType())
                .ruleRegion(ruleDataAddReq.getRuleRegion())
                .ruleCoding(ruleDataAddReq.getRuleCoding())
                .ruleActivate(ruleDataAddReq.getRuleActivate())
                .ruleStatus(ruleDataAddReq.getRuleStatus())
                .ruleDescribe(ruleDataAddReq.getRuleDescribe())
                .build();
    }
}
