package com.muyu.common.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.common.domain.req.RuleVersionAddReq;
import com.muyu.common.domain.req.RuleVersionUpdCodeReq;
import com.muyu.common.domain.req.RuleVersionUpdReq;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.util.function.Supplier;

/**
 * @Author：蓬叁
 * @Package：com.muyu.common.domain
 * @Project：cloud-rule
 * @name：RuleVersion
 * @Date：2024/8/23 上午11:43
 */
@Data
@SuperBuilder
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@TableName(value = "rule_version")
public class RuleVersion extends BaseEntity {

    /**
     * 规则版本ID
     */
    @TableId(value = "rule_version_id",type = IdType.AUTO)
    private Long ruleVersionId ;
    /**
     * 规则ID
     */
    private Long ruleId ;
    /**
     * 版本类
     */
    private String ruleVersionType ;
    /**
     * 版本名称
     */
    private String ruleVersionName ;
    /**
     * 版本编号
     */
    private String ruleVersionCode ;
    /**
     * 是否激活
     */
    private String ruleVersionActivate ;
    /**
     * 版本状态
     */
    private String ruleVersionStatus ;
    /**
     * 是否测试
     */
    private String ruleVersionTest ;
    /**
     * 代码
     */
    private String ruleVersionText ;

    public static RuleVersion addBuild(RuleVersionAddReq ruleVersionAddReq) {
        return RuleVersion.builder()
                .ruleId(ruleVersionAddReq.getRuleId())
                .ruleVersionType(ruleVersionAddReq.getRuleVersionType())
                .ruleVersionName(ruleVersionAddReq.getRuleVersionName())
                .ruleVersionCode(ruleVersionAddReq.getRuleVersionCode())
                .ruleVersionActivate(ruleVersionAddReq.getRuleVersionActivate())
                .ruleVersionStatus(ruleVersionAddReq.getRuleVersionStatus())
                .ruleVersionTest("W")
                .ruleVersionText(ruleVersionAddReq.getRuleVersionText())
                .remark(ruleVersionAddReq.getRemark())
                .build();
    }

    public static RuleVersion updBuild(RuleVersionUpdReq ruleVersionUpdReq, Supplier<Long> idSupplier) {
        return RuleVersion.builder()
                .ruleId(idSupplier.get())
                .ruleVersionActivate(ruleVersionUpdReq.getRuleVersionActivate())
                .build();

    }

    public static RuleVersion updCodeBuild(RuleVersionUpdCodeReq ruleVersionUpdCodeReq, Supplier<Long> idSupplier) {
        return RuleVersion.builder()
                .ruleVersionId(idSupplier.get())
                .ruleVersionText(ruleVersionUpdCodeReq.getRuleVersionText())
                .build();
    }
}
