package com.muyu.load;

import com.aliyun.oss.OSSClient;
import com.muyu.common.core.domain.Result;
import com.muyu.compile.FilesCompilerLoad;
import com.muyu.constant.MethodSuffix;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;

/**
 * @Author：蓬叁
 * @Package：com.muyu.load
 * @Project：cloud-rule
 * @name：OSSFileLoad
 * @Date：2024/8/31 下午6:38
 */
public class OSSFileLoad {

    // 阿里云 endpoint
    private static final String endpoint = "oss-cn-shanghai.aliyuncs.com";
    // 阿里云 accessKeyId
    private static final String accessKeyId = "LTAI5tHKdDATVKeBjFH8mb8D";
    // 阿里云 accessKeySecret　　
    private static final String accessKeySecret = "5ejb4qdgukfD1FdM181kdeAkSuURo7";
    // bucket
    private static final String bucketName = "java-web-generating-class";
    // OSS文件路径
    private static final String fileJavaPath = "build/version/java/";
    private static final String fileClassPath = "build/version/class/";
    private static final Logger log = LoggerFactory.getLogger(OSSFileLoad.class);

    public static Result<Object> streamingJavaDownload(String fileName) throws IOException {
        String source = null;
        // 创建 OSSClient 实例
        OSSClient ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret);
        // 判断 bucket 是否存在
        Boolean flag_bucket = ossClient.doesBucketExist(bucketName);
        if (!flag_bucket) {
            System.out.println("bucket不存在");
            return Result.error("bucket不存在");
        }
        Boolean flag_file = ossClient.doesObjectExist(bucketName, fileJavaPath+fileName+ MethodSuffix.JAVASUFFIX);
        if (!flag_file) {
            System.out.println("预下载文件不存在");
            return Result.error("预下载文件不存在");
        }
        // 本地文件下载路径
        String localPath = "home/" + fileName + MethodSuffix.JAVASUFFIX;

        try {
            // 从OSS下载文件
            InputStream inputStream = ossClient.getObject(bucketName, fileJavaPath + fileName + MethodSuffix.JAVASUFFIX).getObjectContent();
            File downloadFile = new File(localPath);
            FileOutputStream outputStream = new FileOutputStream(downloadFile);

            byte[] buffer = new byte[1024];
            int len;
            while ((len = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, len);
            }
            outputStream.close();
            inputStream.close();

            System.out.println("存放路径:" + localPath);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭OSSClient
            if (ossClient != null) {
                ossClient.shutdown();
            }
        }
        //对路径里的.java文件进行编译
//        SourceCodeCompiler.javaCompilerPath("home/");
//        System.out.println("第二步");
//        File outputDir = new File("home/"); // 或者是你指定的其他输出目录
//        System.out.println("第三步");
//        File[] classFiles = outputDir.listFiles(); // 获取输出目录中的所有文件
//        if (classFiles != null) {
//            for (File classFile : classFiles) {
//                if (classFile.getName().endsWith(".class")) {
//                    System.out.println("找到class文件 " + classFile.getName());
////                    把.class文件存入oss中
//                }
//            }
//        }else {
//            System.out.println("没有找到文件");
//        }
        return Result.success(source);
    }

    public static Result<Object> streamingClassDownload(String fileName) throws IOException {
        String source = null;
        // 创建 OSSClient 实例
        OSSClient ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret);
        // 判断 bucket 是否存在
        Boolean flag_bucket = ossClient.doesBucketExist(bucketName);
        if (!flag_bucket) {
            System.out.println("bucket不存在");
            return Result.error("bucket不存在");
        }
        Boolean flag_file = ossClient.doesObjectExist(bucketName, fileClassPath+fileName+ MethodSuffix.CLASSSUFFIX);
        if (!flag_file) {
            System.out.println("预下载文件不存在");
            return Result.error("预下载文件不存在");
        }
        // 本地文件下载路径
        String localPath = "/home/lib/com/muyu/generate/" + fileName + MethodSuffix.CLASSSUFFIX;

        try {
            log.info("执行下载class文件:[{}]",localPath);
            // 从OSS下载文件
            InputStream inputStream = ossClient.getObject(bucketName, fileClassPath + fileName + MethodSuffix.CLASSSUFFIX).getObjectContent();
            File downloadFile = new File(localPath);
            log.info("下载地址:[{}]",downloadFile);
            FileOutputStream outputStream = new FileOutputStream(downloadFile);

            byte[] buffer = new byte[1024];
            int len;
            while ((len = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, len);
            }
            outputStream.close();
            inputStream.close();

            System.out.println("存放路径:" + localPath);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭OSSClient
            if (ossClient != null) {
                ossClient.shutdown();
            }
        }

        return Result.success(source);
    }

}
