package com.muyu.servier.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.enums.SysRuleActivate;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.domain.DataRecord;
import com.muyu.common.domain.DataValue;
import com.muyu.common.domain.RuleData;
import com.muyu.common.domain.RuleVersion;
import com.muyu.common.domain.req.RuleVersionUpdReq;
import com.muyu.common.domain.resp.RuleVersionResp;
import com.muyu.compile.FilesCompilerLoad;
import com.muyu.constant.GenerateConstant;
import com.muyu.constant.MethodSuffix;
import com.muyu.engine.basic.BasicEngine;
import com.muyu.load.OSSFileLoad;
import com.muyu.mapper.RuleVersionMapper;
import com.muyu.servier.RuleDataService;
import com.muyu.servier.RuleVersionService;
import com.muyu.upload.ALiYunUpload;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Author：蓬叁
 * @Package：com.muyu.servier.impl
 * @Project：cloud-rule
 * @name：RuleVersionServiceImpl
 * @Date：2024/8/27 上午11:47
 */
@Log4j2
@Service
public class RuleVersionServiceImpl
        extends ServiceImpl<RuleVersionMapper, RuleVersion>
        implements RuleVersionService {

    @Autowired private RuleVersionMapper ruleVersionMapper;
    @Autowired private RuleDataService ruleDataService;

    public static final String ClassPath = "/home/lib/com/muyu/generate/";

    static Map<String , BasicEngine<DataValue>> engineValueMap = new ConcurrentHashMap<>();
    static Map<String , BasicEngine<DataRecord>> engineRecordMap = new ConcurrentHashMap<>();

    @Override
    public List<RuleVersionResp> selectList(Long ruleId) {
        List<RuleVersionResp> ruleVersion = ruleVersionMapper.selectByIdList(ruleId);
        return ruleVersion;
    }

    @Override
    public void updateStatus(Long ruleVersionId, RuleVersionUpdReq ruleVersionUpdReq) {
        ruleVersionMapper.updateStatus(ruleVersionId,ruleVersionUpdReq);
    }

    @Override
    public void unstart(Long ruleVersionId) {
        this.settingVersionActivate(ruleVersionId, SysRuleActivate.UNSTART.getCode());
    }

    @Override
    public void start(Long ruleVersionId) {
        this.settingVersionActivate(ruleVersionId, SysRuleActivate.START.getCode());
    }

    @Override
    public void settingVersionActivate(Long ruleVersionId, String activate) {
        LambdaQueryWrapper<RuleVersion> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleVersion::getRuleVersionId,ruleVersionId);
        boolean isExists = this.exists(queryWrapper);
        if (!isExists) {
            throw new ServiceException("操作规则不存在");
        }
        if (!SysRuleActivate.isCode(activate)){
            throw new ServiceException("设置状态值违法");
        }
        LambdaUpdateWrapper<RuleVersion> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(RuleVersion::getRuleVersionId,ruleVersionId);
        updateWrapper.set(RuleVersion::getRuleVersionActivate,activate);
        this.update(updateWrapper);
    }

    @Override
    public RuleVersion generatedCode(RuleVersion ruleVersion) {
        RuleData ruleData = ruleDataService.getById(ruleVersion.getRuleId());
        ruleVersion.setRuleVersionType(getClassName("generate"+"_"+ruleData.getRuleCoding()+"_"+ruleVersion.getRuleVersionCode()));
        ruleVersion.setRuleVersionText(GenerateConstant.generateConstant(ruleData,ruleVersion));
        return ruleVersion;
    }

    @Override
    public void rowTest(DataRecord dataRecord) {

        //上传java文件
        ALiYunUpload.uploadJavaFiles(dataRecord.getRuleVersionTest(),dataRecord.getRuleVersionType());

        try {
            System.out.println("执行编译");
            OSSFileLoad.streamingJavaDownload(dataRecord.getRuleVersionType());
        } catch (IOException e) {
            log.info(e.getMessage());
        }

        //编译class
        FilesCompilerLoad.javaCompiler(dataRecord.getRuleVersionType());

        //上传class
        ALiYunUpload.uploadClassFiles(
                dataRecord.getRuleVersionType(),
                ClassPath+dataRecord.getRuleVersionType()+ MethodSuffix.CLASSSUFFIX);
        log.info("开始拉取....");

        //下载class
        try {
            OSSFileLoad.streamingClassDownload(dataRecord.getRuleVersionType());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        //对路径里的.class文件进行编译
        log.info("执行加载class:[{}]",dataRecord.getRuleVersionType());
        Result<Class<?>> classResult = FilesCompilerLoad.classLoad(dataRecord.getRuleVersionType());
        Class<?> clazz = classResult.getData();

        try {
            Object instance = clazz.getDeclaredConstructor().newInstance();


            engineRecordMap.put(dataRecord.getRuleVersionType(), (BasicEngine<DataRecord>) instance);

        } catch (InstantiationException | IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
            throw new RuntimeException(e);
        }

        DataValue[] dataValues = new DataValue[3];
        dataValues[0].setKey("id");
        dataValues[0].setType("bigint");
        dataValues[0].setLabel("主键ID");
        dataValues[0].setValue("1");
        dataValues[1].setKey("name");
        dataValues[1].setType("string");
        dataValues[1].setLabel("名字");
        dataValues[1].setValue("张三");
        dataValues[2].setKey("age");
        dataValues[2].setType("int");
        dataValues[2].setLabel("年龄");
        dataValues[2].setValue("20");

        dataRecord.setDataValues(dataValues);

        BasicEngine<DataRecord> engine = engineRecordMap.get(dataRecord.getRuleVersionType());
        engine.set(dataRecord.getDataValues()[3]);
        engine.execution();

    }

    public static String getClassName(String versionCode){
        String[] splits = versionCode.split("_");
        String className="";
        for (String split : splits) {
            className += split.substring(0,1).toUpperCase()+split.substring(1);
        }
        return className;
    }

}
